<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\GER\Utilities\MessagesProcedureInflightor;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class PlanejamentoApresentacaoCheftableController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
	protected $dataSourceName = "planapreinf_manualciam";
	protected $dataSourceNameToSave  = "planapreinf";
    protected $crudService;
    protected $dataSourceManager;
    protected $parameterBagColumns = array();
    protected $rows;
    protected $entityManager;
    protected $novoCodigo;
    protected $MessagesProcedureInflightor;
    
    public function __construct($crudService, ParameterBag $parameterBag, $dataSourceManager, $novoCodigo, $entityManager, $MessagesProcedureInflightor){
		$this->dataSourceManager = $dataSourceManager;
		$this->novoCodigo = $novoCodigo;
		$this->entityManager = $entityManager;
		$this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
		parent::__construct($crudService,$parameterBag);
    }
    
    public function deleteApresentacaoESubgrupos(DTO\Request\Row $request, DTO\Response $response){
        $row = $request->getRow();
        if($row['IDSITUACAO'] != 'APROVADO'){
            $this->deleteSubgrupos($row, $response);
            $this->deleteApresentacao($request, $response);
        }else{
            $response->setError(new Error('Operação bloqueada. Apresentação já está Aprovada.', 500));
        }
        
    }
    
    private function deleteSubgrupos($row, $response){
        $resultSet = $this->findSubgrupos($row);
        $dataSet = new DataSet("", $resultSet);
        $dataSet->setDataSourceName("itplanapreinf");
        $this->dataSourceManager->delete($dataSet);
    }
    
    private function deleteApresentacao($request, $response){
        $request = $this->crudService->buildDataSetIfRow($request, $this->dataSourceName);
        $dataSet = $request->getDataSet();
        $dataSet->setDataSourceName("planapreinf");
        $this->dataSourceManager->delete($dataSet);
    }
    
    private function findSubgrupos($row){
        $filtro = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $row['CDFILIAL']
            ),
            array(
                "columnName" => "NRPLANAPREINF",
                "operator" => "=",
                "value" => $row['NRPLANAPREINF']
            )
        );
        
        $newFilterCriteria = new FilterCriteria("itplanapreinf", $filtro);
        $dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
        return $dataSet->getRows();
    }
    
    public function salvaPlanejamento(DTO\Request\Row $request, DTO\Response $response){
        try {
            $dataSourceName = $this->dataSourceNameToSave ? $this->dataSourceNameToSave : $this->dataSourceName;
            $request->getRow();
            $row = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $row);
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
    }
    
    public function saveChanges(DTO\Request\Row $request, DTO\Response $response){
	    $connection = $this->entityManager->getConnection();
        foreach ($request->getRow() as &$row) {
            $flag = 'save';
            if(isset($row['NRITPLANAPREINF'])){
                $row['__is_new'] = false;
                if($row['QTSUBGRU'] == 0) {
                    $flag = 'delete';
                }
            } else {
                $row['NRITPLANAPREINF'] = $this->novoCodigo->getCodeByTableName('ITPLANAPREINF'.$this->crudService->getEnvironment()->getCdFilial(), $this->crudService->getEnvironment()->getNrOrg());
                $row['__using_unit']  = true;
                $row['__is_new_code']  = true;
                $row['__new_code_pk '] = "ITPLANAPREINF";
                $row['__table_pk '] = "NRITPLANAPREINF";
                $row['__new_code_length']  = 20;
                $row['__is_new'] = true;
            }
            if ($flag == 'save') {
                $newDataSet = new DataSet('itplanapreinf', array($row));
        	    $newRequest = new \Zeedhi\Framework\DTO\Request\DataSet($newDataSet,
                $request->getMethod(), $request->getRoutePath(), $request->getUserId());
    	    	$rows = $this->crudService->getRowsToSave($newRequest, 'itplanapreinf');
                $this->crudService->save('itplanapreinf', $rows);
            } else {
    			$connection->beginTransaction();
                try {
                    $newDataSet = new DataSet('itplanapreinf', array($row));
    			    $this->dataSourceManager->delete($newDataSet);
        			$connection->commit();
                } catch(\Exception $e) {
        			$response->setError(new Error($e->getMessage(), 500));
        			$connection->rollback();
        		}
            }
        }
    }
    
    public function callProcedurePlana(DTO\Request\Row $request, DTO\Response $response){
	    $connection = $this->entityManager->getConnection();
        try {
			$connection->beginTransaction();
            $row = $request->getRow();
            if ($row['PROC'] == 'A'){
                $aprovaPlana = $this->aprovaPlana($row['CDFILIAL'], $row['NRPLANAPREINF'], $this->crudService->getEnvironment()->getCdOperador(), $row['NRORG']);
                if($aprovaPlana != null) {
                    $this->MessagesProcedureInflightor->concatErrorMessages($row['CDFILIAL'],$aprovaPlana, $response);
                    $connection->rollback();
                    return;
                }
                $connection->commit();
            } else {
                $cancelaPlana = $this->cancelaPlana($row['CDFILIAL'], $row['NRPLANAPREINF'], $this->crudService->getEnvironment()->getCdOperador(), $row['NRORG']);
                if($cancelaPlana != null) {
                    $this->MessagesProcedureInflightor->concatErrorMessages($row['CDFILIAL'], $cancelaPlana, $response);
                    $connection->rollback();
                    return;
                }
                $connection->commit();
            }
        } catch (\Exception $e) {
            $response->setError(new Error($this->MessagesProcedureInflightor->convertMessage($e->getMessage()), 500));
            $connection->rollback();
        }
    }
    
    public function aprovaPlana($cdfilial, $nrplanapreinf, $cdoperador, $nrorg){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'APROVA_PLANAPREINF');
        $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('P_NRPLANAPREINF',Param::PARAM_INPUT, $nrplanapreinf));
        $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador));
        $procedure->addParam(new Param('P_NRORG',        Param::PARAM_INPUT, $nrorg));
        $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_INT, 20));
        $result = $procedure->execute();
        if (isset($result['P_NRLISTAERROS'])) {
            return $result['P_NRLISTAERROS'];
        }
        return [];
    }
    
    public function cancelaPlana($cdfilial, $nrplanapreinf, $cdoperador, $nrorg){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'CANCELA_APROV_PLANAPREINF');
        $procedure->addParam(new Param('P_CDFILIAL',     Param::PARAM_INPUT, $cdfilial));
        $procedure->addParam(new Param('P_NRPLANAPREINF',Param::PARAM_INPUT, $nrplanapreinf));
        $procedure->addParam(new Param('P_CDOPERADOR',   Param::PARAM_INPUT, $cdoperador));
        $procedure->addParam(new Param('P_NRORG',        Param::PARAM_INPUT, $nrorg));
        $procedure->addParam(new Param('P_NRLISTAERROS', Param::PARAM_OUTPUT, '', Param::PARAM_TYPE_INT, 20));
        $result = $procedure->execute();
        if (isset($result['P_NRLISTAERROS'])) {
            return $result['P_NRLISTAERROS'];
        }
        return [];
    }

}
