<?php

namespace Controllers;

use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DTO\Response\Message;
use \Zeedhi\Framework\DataSource\DataSet;
use \Zeedhi\Framework\DataSource\FilterCriteria;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class ProducaoSetor extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    const RECEITAS_CHECKLIST = 'receitas_do_checklist';
    const DESTINOS_CHECKLIST = 'destino_receita_checklist';
    
    protected $environment; 
    protected $dataSourceManager;
    protected $crudService;
    protected $rateioservice;
    protected $parameterBag = array();
    protected $connection;
    protected $entityManager;
    protected $MessagesProcedureInflightor;

    public function __construct($crudService, $parameterBag, $environment, $dataSourceManager, $rateioservice, $connection, $entityManager, $MessagesProcedureInflightor) {
        $this->crudService       = $crudService;
        $this->rateioservice     = $rateioservice;
        $this->parameterBag      = $parameterBag;
        $this->environment       = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection        = $connection;
        $this->MessagesProcedureInflightor = $MessagesProcedureInflightor;
        $this->entityManager     = $entityManager;
    }
    
    public function inventariosZerados(DTO\Request\DataSet $request, DTO\Response $response){
        $parametros = array();
        $rows = $request->getDataSet()->getRows();
        $sql = "SELECT RCKL.NRRECCKLISTPRODINF, RCKL.QTRECEITAEST AS QTDRECEITA,
                      DCKL.NRDESTCKLISTPRODINF, DCKL.QTRECEITAEST 
                FROM RECCKLISTPRODINF RCKL
                LEFT JOIN DESTCKLISTPRODINF DCKL
                ON  DCKL.CDFILIAL = RCKL.CDFILIAL
                AND DCKL.NRRECCKLISTPRODINF = RCKL.NRRECCKLISTPRODINF
                WHERE RCKL.CDFILIAL = :CDFILIAL
                AND   RCKL.NRCKLISTPRODINF = :NRCKLISTPRODINF";
        $errorMessage = 0;
        foreach ($rows as $row) {
            $parametros['CDFILIAL']    = $row['CDFILIAL'];
            $parametros['NRCKLISTPRODINF']  = $row['NRCKLISTPRODINF'];
            $result = $this->connection->fetchAll($sql, $parametros);
            if(!empty($result)){
                foreach ($result as $receita) {
                    if($receita['QTDRECEITA'] == 0 || $receita['QTRECEITAEST'] == 0){
                        $errorMessage++;
                    }
                }
            }
        }
        $msgErro = array();
        if($errorMessage > 0){
            array_push($msgErro, "PCP00027_CONF_EXISTE_CKLIST_SALDO_ZERO");
        }
        $response->addDataSet(new DataSet("inventarios_zerados", $msgErro));
    }

    public function salvaAlteracoesProducaoSetor(DTO\Request\DataSet $request, DTO\Response $response){
        
        $rows = current($request->getDataSet()->getRows());
        $justificativas = $rows['justificativa'];
        $receitas = $rows['receitas'];
        $origem = $rows['origem'];
        
        try {
            $this->connection->beginTransaction();
            if(self::RECEITAS_CHECKLIST === $origem){
                
                if(sizeof($justificativas) > 0){
                    $justificativas = $this->limpaJustificativasExistentes($justificativas, $receitas);
                    $this->salvaJustificativa($justificativas, $receitas);
                } 
                
                $this->salvaAlteracoesReceitasCheckList($receitas,'producao');
                $response->addMessage(new Message("PCP00027_ALT_SALVA"));
            }else if(self::DESTINOS_CHECKLIST === $origem){
                if(sizeof($justificativas) > 0){
                    
                    $justificativas = $this->limpaJustificativasExistentes($justificativas, $receitas);
                    $this->salvaJustificativa($justificativas, $receitas);
                }
                
                $this->salvaAlteracoesDestinoCheckList($receitas, $response);
                $this->aplicaProducaoRejDesperdicioReceita($receitas, 'producao');
                $response->addMessage(new Message("PCP00027_ALT_SALVA"));
            }
            $this->connection->commit();
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }

    }
    
    private function salvaJustificativa($justificativas, $dataset){
        if(sizeof($justificativas)>0){
            $novoDatasetJustificativas = $this->montaDatasetJustificativas($justificativas, $dataset);
            $novoDataset = new DataSet('jusrejdesp', $novoDatasetJustificativas);
    		$novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
    		$justificativasParaSalvar = $this->crudService->getRowsToSave($novoRequest, "jusrejdesp");
    		$dataSetParaSalvar = new DataSet('jusrejdesp', $justificativasParaSalvar);
    		$this->dataSourceManager->persist($dataSetParaSalvar);
        }
    }
    
    private function montaDatasetJustificativas($justificativas, $dataset){
        $novoDataset = array();
        //verificar justificativas ja existentes e ignora elas
        $justificativas = $this->limpaJustificativasExistentes($justificativas, $dataset);
        foreach($justificativas as $justificativa){
            foreach($dataset as $produto){
                array_push($novoDataset, array(
                    "CDFILIAL"=> $produto['CDFILIAL'],
                    "NRRECCKLISTPRODINF"=> $produto['NRRECCKLISTPRODINF'],
                    "NRJUSTIFICATIVAPROD"=> $justificativa['NRJUSTIFICATIVAPROD'],
                    "__is_new"=> true,
                    "changed"=> true
                ));
            }
        }
        return $novoDataset;
    }
    
    private function salvaAlteracoesReceitasCheckList($receitas, $origemDaTela){
        $novoDataset = new DataSet('reccklistprodinf', $receitas);
		$novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
		$receitasParaSalvar = $this->crudService->getRowsToSave($novoRequest, "reccklistprodinf");
		$dataSetParaSalvar = new DataSet('reccklistprodinf', $receitasParaSalvar);
		$this->dataSourceManager->persist($dataSetParaSalvar);
        foreach ($receitas as $receita) {
            $destinosReceita = $this->retornaDestinosReceita($receita)->getRows();
            if(sizeof($destinosReceita) > 0){
                $destinosReceita = $this->aplicaValoresBaseadosEmSuasReceitas($receita, $destinosReceita, $origemDaTela);
                $this->salvaAlteracoesDestinoCheckList($destinosReceita);
            }
        }
        
    }
    
    private function salvaAlteracoesDestinoCheckList($destinosReceita){
        
        $novoDataset = new DataSet('destcklistprodinf', $destinosReceita);
		$novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
		$destinosParaSalvar = $this->crudService->getRowsToSave($novoRequest, "destcklistprodinf");
		$dataSetParaSalvar = new DataSet('destcklistprodinf', $destinosParaSalvar);
		$this->dataSourceManager->persist($dataSetParaSalvar);
        
    }
    
    private function aplicaProducaoRejDesperdicioReceita($destinosReceita, $telaOrigem){
        
        $receitaPai = $this->retornaReceitaPai($destinosReceita);
        $receitaPai = $this->somaQuantidadeDestinos($receitaPai, $telaOrigem);
        $novoDataset = new DataSet('reccklistprodinf', $receitaPai);
		$novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
		$receitasParaSalvar = $this->crudService->getRowsToSave($novoRequest, "reccklistprodinf");
		$dataSetParaSalvar = new DataSet('reccklistprodinf', $receitasParaSalvar);
		$this->dataSourceManager->persist($dataSetParaSalvar);
		
    }
    
    private function somaQuantidadeDestinos($receitaPai, $telaOrigem){
        $soma = array(
            'QTRECEITAREJDESP' => 0,
            'QTRECEITAPROD' => 0,
            'QTRECEITAINF' => 0,
            'QTRECEITAEST' => 0
        );
        
        $destinosReceita = $this->retornaDestinosReceita($receitaPai[0])->getRows();
        
        if($telaOrigem == 'producao'){
            foreach ($destinosReceita as $destino) {
                if(isset($destino['QTRECEITAREJDESP'])){
                    $soma['QTRECEITAREJDESP'] += $destino['QTRECEITAREJDESP'];
                }
                if(isset($destino['QTRECEITAPROD'])){
                    $soma['QTRECEITAPROD'] += $destino['QTRECEITAPROD'];
                }
            }
            if(isset($destino['QTRECEITAREJDESP'])){
                $receitaPai[0]['QTRECEITAREJDESP'] = $soma['QTRECEITAREJDESP'];
            }
            if(isset($destino['QTRECEITAPROD'])){
                $receitaPai[0]['QTRECEITAPROD'] = $soma['QTRECEITAPROD'];
            }
            $receitaPai[0]['DTULTATU'] = null;
            $receitaPai[0]['CDOPERULTATU'] = null;
            $receitaPai[0]['__is_new'] = false;
            $receitaPai[0]['changed'] = true;
        }
        if($telaOrigem == 'planejamento'){
            foreach ($destinosReceita as $destino) {
                if(isset($destino['QTRECEITAINF'])){
                    $soma['QTRECEITAINF'] = $soma['QTRECEITAINF'] + $destino['QTRECEITAINF'];
                }
                if(isset($destino['QTRECEITAEST'])){
                    $soma['QTRECEITAEST'] = $soma['QTRECEITAEST'] + $destino['QTRECEITAEST'];
                }
            }
            if(isset($destino['QTRECEITAINF'])){
                $receitaPai[0]['QTRECEITAINF'] = $soma['QTRECEITAINF'];
            }
            if(isset($destino['QTRECEITAEST'])){
                $receitaPai[0]['QTRECEITAEST'] = $soma['QTRECEITAEST'];
            }
            $receitaPai[0]['DTULTATU'] = null;
            $receitaPai[0]['CDOPERULTATU'] = null;
            $receitaPai[0]['__is_new'] = false;
            $receitaPai[0]['changed'] = true;
        }
        return $receitaPai;
    }
    
    private function retornaReceitaPai($destinosReceita){
        $filtro = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $destinosReceita[0]["CDFILIAL"]
            ),
            array(
                "columnName" => "NRRECCKLISTPRODINF",
                "operator" => "=",
                "value" => $destinosReceita[0]["NRRECCKLISTPRODINF"]
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $destinosReceita[0]["NRORG"]
            )
        );
        
        $filterCriteria = new FilterCriteria('reccklistprodinf', $filtro);
        $novoFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($novoFilterCriteria);
        return $dataSet->getRows();
    }
    
    private function aplicaValoresBaseadosEmSuasReceitas($receita, $destinosReceita, $origemDaTela){
        //aplica rateio
        if($origemDaTela == 'producao'){
            if(isset($receita["QTRECEITAREJDESP"])){
                $this->rateioservice->setOriginColumns("QTRECEITAREJDESP");
                $this->rateioservice->setReferenceColumns("QUANTIDADE");
                $this->rateioservice->setAffectedColumns("QTRECEITAREJDESP");
                $destinosNormalizados = $this->rateioservice->calcRateio($receita, $destinosReceita);
                foreach ($destinosNormalizados as &$destino) {
                    $destino['DTULTATU'] = null;
                    $destino['CDOPERULTATU'] = null;
                    $destino['changed'] = true;
                }
            }
            if(isset($receita["QTRECEITAPROD"])){
                $this->rateioservice->setOriginColumns("QTRECEITAPROD");
                $this->rateioservice->setReferenceColumns("QUANTIDADE");
                $this->rateioservice->setAffectedColumns("QTRECEITAPROD");
                $destinosNormalizados = $this->rateioservice->calcRateio($receita, $destinosReceita);
                foreach ($destinosNormalizados as &$destino) {
                    $destino['DTULTATU'] = null;
                    $destino['CDOPERULTATU'] = null;
                    $destino['changed'] = true;
                }
            }
        }
        if($origemDaTela == 'planejamento'){
            if(isset($receita["QTRECEITAINF"])){
                $this->rateioservice->setOriginColumns("QTRECEITAINF");
                $this->rateioservice->setReferenceColumns("QTRECEITAINF");
                $this->rateioservice->setAffectedColumns("QTRECEITAINF");
                $destinosNormalizados = $this->rateioservice->calcRateio($receita, $destinosReceita);
                foreach ($destinosNormalizados as &$destino) {
                    $destino['DTULTATU'] = null;
                    $destino['CDOPERULTATU'] = null;
                    $destino['changed'] = true;
                }
            }
            if(isset($receita["QTRECEITAEST"])){
                $this->rateioservice->setOriginColumns("QTRECEITAEST");
                $this->rateioservice->setReferenceColumns("QTRECEITAINF");
                $this->rateioservice->setAffectedColumns("QTRECEITAEST");
                $destinosNormalizados = $this->rateioservice->calcRateio($receita, $destinosReceita);
                foreach ($destinosNormalizados as &$destino) {
                    $destino['DTULTATU'] = null;
                    $destino['CDOPERULTATU'] = null;
                    $destino['changed'] = true;
                }
            }
        }
        
        return $destinosNormalizados;
        
    }
    
    private function retornaDestinosReceita($receita){
        
        $filtro = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $receita["CDFILIAL"]
            ),
            array(
                "columnName" => "NRRECCKLISTPRODINF",
                "operator" => "=",
                "value" => $receita["NRRECCKLISTPRODINF"]
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $receita["NRORG"]
            )
        );
        
        $filterCriteria = new FilterCriteria('destino_receita', $filtro);
        $novoFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($novoFilterCriteria);
        
        return $dataSet;
        
    }  
    
    private function retornaJustificativas($justificativa, $receitas){
        
        $filtro = array(
            array(
                "columnName" => "CDFILIAL",
                "operator" => "=",
                "value" => $receitas[0]["CDFILIAL"]
            ),
            array(
                "columnName" => "NRRECCKLISTPRODINF",
                "operator" => "=",
                "value" => $receitas[0]["NRRECCKLISTPRODINF"]
            ),
            array(
                "columnName" => "NRORG",
                "operator" => "=",
                "value" => $receitas[0]["NRORG"]
            ),
            array(
                "columnName" => "NRJUSTIFICATIVAPROD",
                "operator" => "=",
                "value" => $justificativa['NRJUSTIFICATIVAPROD']
            )
        );
        
        $filterCriteria = new FilterCriteria('jusrejdesp', $filtro);
        $novoFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $dataSet = $this->dataSourceManager->findBy($novoFilterCriteria);
        return $dataSet->getRows();
        
    }
    
    private function limpaJustificativasExistentes($justificativas, $receitas){
        
        $novasJustificativas = array();
        
        foreach($justificativas as $justificativa){
            if(empty($this->retornaJustificativas($justificativa, $receitas)))
                array_push($novasJustificativas, $justificativa);
        }
        return $novasJustificativas;
        
    }
    
    public function retornaCalculo(DTO\Request $request, DTO\Response $response){
        $filtros = $request->getFilterCriteria()->getConditions();
        $parametros = array();
        
        foreach($filtros as $filtro){
           $parametros[$filtro['columnName']] = $filtro['value'];
        }
        
        $retornaPametrosReceita = $this->retornaPametrosReceita($parametros);
        $varNrsequtilrecflex = $this->retornaNrsequtilrecflex($parametros);
        
        $sql = "SELECT 'P' AS TIPO, 
                       PO.CDPRODUTO AS CHAVE, 
                       PO.CDARVPROD AS CODIGO, 
                       PO.NMPRODUTO AS NOME, 
                       ROUND(:FATOR * NVL(PR.QTPRODREC, 0), 3) AS QUANTIDADE, 
                       PO.SGUNIDADE, 
                       PR.NRORDPREC AS ORDEM
                  FROM PRORECEITAM PR
                 INNER JOIN PRODUTO PO 
                    ON PO.CDPRODUTO = PR.CDPRODUTO
                 WHERE PR.CDRECEITA = :CDRECEITA
                   AND PR.NRVARIACAOREC = :NRVARIACAOREC
                   AND (:IDRECFLEXIVEL = 'N' OR 
                        (COMPOSICAO_EM_PRODUCAO(:CDFILIAL, 
                                                :CDRECEITA, 
                                                :NRORG, 
                                                :NRSEQUTILRECFLEX, 
                                                PR.CDPRODUTO, 
                                                'P') = 'S') OR
                        (:NRSEQUTILRECFLEX IS NULL AND 
                        PR.IDPREFERENCIA = 'S'))
                 UNION ALL
                SELECT 'R' AS TIPO,
                       SR.CDSUBRECEITA AS CHAVE, 
                       NVL(R.CDARVPROD, R.CDRECEITA) AS CODIGO, 
                       R.DSRECEITA AS NOME, 
                       ROUND(:FATOR * NVL(SR.QTRENDSUBREC,0), 3) AS QUANTIDADE, 
                       R.SGUNIDADE, 
                       SR.NRORDSUBREC AS ORDEM
                  FROM SUBRECEITAM SR
                 INNER JOIN RECEITAM R ON R.CDRECEITA = SR.CDSUBRECEITA
                 WHERE SR.CDRECEITA = :CDRECEITA
                   AND SR.NRVARIACAOREC = :NRVARIACAOREC
                   AND (:IDRECFLEXIVEL = 'N' OR 
                        (COMPOSICAO_EM_PRODUCAO(:CDFILIAL, 
                                                :CDRECEITA, 
                                                :NRORG, 
                                                :NRSEQUTILRECFLEX, 
                                                SR.CDSUBRECEITA, 
                                                'S') = 'S') OR
                        (:NRSEQUTILRECFLEX IS NULL AND SR.IDPREFERENCIA = 'S'))
                 ORDER BY ORDEM, NOME";
        
        $parametros['NRVARIACAOREC']    = $retornaPametrosReceita['NRVARIACAOREC'];
        $parametros['QTRENDIMENTOREC']  = $retornaPametrosReceita['QTRENDIMENTOREC'];
        $parametros['NROPCOES']         = $retornaPametrosReceita['NROPCOES'];
        $parametros['NRSEQUTILRECFLEX'] = $varNrsequtilrecflex['NRSEQUTILRECFLEX'];
        $parametros['FATOR']            = $retornaPametrosReceita['QTRENDIMENTOREC'] == 0 ? 0 : ($parametros['QUANTIDADE']/$retornaPametrosReceita['QTRENDIMENTOREC']);
        $parametros['IDRECFLEXIVEL']    = $retornaPametrosReceita['IDRECFLEXIVEL'];
        $parametros['NRORG']            = $this->environment->getNrOrgTrab();
        
        $params = $parametros;
        $result = $this->connection->fetchAll($sql, $params);
        $dataSet = new DataSet('composicao_calculo_receita_produzir', $result);
        $response->addDataSet($dataSet);
        
    }
    
    private function retornaPametrosReceita($parametros){
        
        $sql = "SELECT V.NRVARIACAOREC, 
                       NVL(V.QTRENDIMENTOREC, 0) AS QTRENDIMENTOREC, 
                       V.NROPCOES,
                       R.IDRECFLEXIVEL
                  FROM FILIRECEITA F 
                 INNER JOIN RECEITAM R
                    ON R.CDRECEITA = F.CDRECEITA
                 INNER JOIN VARIACAOREC V 
                    ON V.CDRECEITA = F.CDRECEITA 
                   AND V.NRVARIACAOREC = F.NRVARIACAOREC 
                 WHERE F.CDRECEITA = :CDRECEITA 
                   AND F.CDFILIAL = :CDFILIAL";
        
        $params = $parametros;
        $result = $this->connection->fetchAll($sql, $params);
		
		return $result[0];
        
    }
    
    private function retornaNrsequtilrecflex($parametros){
        
        $sql = "SELECT MAX(NRSEQUTILRECFLEX) AS NRSEQUTILRECFLEX
                  FROM UTILRECFLEX
                 WHERE CDFILIAL = :CDFILIAL
                   AND CDRECEITA = :CDRECEITA
                   AND :DTPRODUCAO BETWEEN DTINIVIGENCIA AND DTFIMVIGENCIA";
        
        $params = $parametros;
        $result = $this->connection->fetchAll($sql, $params);
		
		return $result[0];
        
    }
    
    public function salvaAlteracoesPlanejamentoProducaoSetor(DTO\Request\DataSet $request, DTO\Response $response){
        
        $rows = current($request->getDataSet()->getRows());
        $receitas = $rows['receitas'];
        $origem = $rows['origem'];
        
        try {
            $this->connection->beginTransaction();
            if(self::RECEITAS_CHECKLIST === $origem){
                
                $this->salvaAlteracoesReceitasCheckList($receitas, 'planejamento');
                $response->addMessage(new Message("PCP00027_ALT_SALVA"));
            }else if(self::DESTINOS_CHECKLIST === $origem){
                
                $this->salvaAlteracoesDestinoCheckList($receitas);
                $this->aplicaProducaoRejDesperdicioReceita($receitas, 'planejamento');
                $response->addMessage(new Message("PCP00027_ALT_SALVA"));
            }
            $this->connection->commit();
        } catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }
        
    }
    
    public function aprovacaoPlanejamentoProducao(DTO\Request\DataSet $request, DTO\Response $response){
        
        $checklistsRows = $request->getDataSet()->getRows();
        
        foreach ($checklistsRows as &$checklist) {
            $checklist['IDSITUACAO'] = 'APROVADOPROD';
            $checklist['changed'] = true;
            $checklist['__is_new'] = false;
        }
        
        try {
			$dataSourceName = 'cklistprodinf';
			$novoDataset = new DataSet($dataSourceName, $checklistsRows);
		    $novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
        
    }
    
    public function cancelaAprovacaoPlanejamentoProducao(DTO\Request\DataSet $request, DTO\Response $response){
        
        $checklistsRows = $request->getDataSet()->getRows();
        
        foreach ($checklistsRows as &$checklist) {
            $checklist['IDSITUACAO'] = 'APROVADOMRP';
            $checklist['changed'] = true;
            $checklist['__is_new'] = false;
        }
        
        try {
			$dataSourceName = 'cklistprodinf';
			$novoDataset = new DataSet($dataSourceName, $checklistsRows);
		    $novoRequest = new DTO\Request\DataSet($novoDataset, '', '', '');
			$rows = $this->crudService->getRowsToSave($request, $dataSourceName);
			$this->crudService->save($dataSourceName, $rows);
            $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
        
    }
    
    public function gerarRequisicao(DTO\Request $request, DTO\Response $response){
        
        $checklistsRows = $request->getDataSet()->getRows();
        try {
            $resultados = array();
            $arrayErros = array();
            $this->connection->beginTransaction();
    		foreach ($checklistsRows as $row) {
    	        $connection = $this->entityManager->getConnection();
    	        $procedure = new StoredProcedure($connection, 'GERA_CKLIST_REQUISICAOINF');
    	        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT,  $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
    	        $procedure->addParam(new Param('P_NRCKLISTPRODINF', Param::PARAM_INPUT,  $row['NRCKLISTPRODINF'],             Param::PARAM_TYPE_INT));
    	        $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT,  $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
    	        $procedure->addParam(new Param('P_NRORG',           Param::PARAM_INPUT,  $row['NRORG'],                       Param::PARAM_TYPE_INT));
    	        $procedure->addParam(new Param('P_NRLISTAERROS',    Param::PARAM_OUTPUT, '',                                  Param::PARAM_TYPE_INT, 20));
    	        $resultado = $procedure->execute();
    	        if($resultado['P_NRLISTAERROS'] != null) {
                    $erros = array();
                    $erros = $this->MessagesProcedureInflightor->getErrorMessages($row['CDFILIAL'], $resultado['P_NRLISTAERROS'], $response, null, false);
                    if(!empty($erros)){
                        array_push($arrayErros, $erros);
                    }
                } 
                array_push($resultados, $resultado);
    	    }
    	    if(!empty($arrayErros) && count($arrayErros)){
    	        $response->addDataSet(new DataSet("ListaErros", $erros));
                $this->connection->rollback();
    	    } else {
        	    $response->addMessage(new Message('Requisições geradas.'));
                $this->connection->commit();
    	    }
        } catch (\Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
        
	}
	
	public function cancelaRequisicao(DTO\Request\DataSet $request, DTO\Response $response){
        
        $checklistsRows = $request->getDataSet()->getRows();
        try {
            $this->connection->beginTransaction();
    		$resultados = array();
            foreach ($checklistsRows as $row) {
    	        $connection = $this->entityManager->getConnection();
    	        $procedure = new StoredProcedure($connection, 'CANCELA_CKLIST_REQUISICAOINF');
    	        $procedure->addParam(new Param('P_CDFILIAL',        Param::PARAM_INPUT,  $row['CDFILIAL'],                    Param::PARAM_TYPE_STR));
    	        $procedure->addParam(new Param('P_NRCKLISTPRODINF', Param::PARAM_INPUT,  $row['NRCKLISTPRODINF'],             Param::PARAM_TYPE_INT));
    	        $procedure->addParam(new Param('P_CDOPERADOR',      Param::PARAM_INPUT,  $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
    	        $procedure->addParam(new Param('P_NRORG',           Param::PARAM_INPUT,  $row['NRORG'],                       Param::PARAM_TYPE_INT));
    	        $resultado = $procedure->execute();
                array_merge($resultados, $resultado);   
            }
            $this->connection->commit();
    	    $response->addDataset(new DataSet('response', $resultados));
        } catch (\Exception $e){
            $this->connection->rollback();
            $response->setError(new Error($this->MessagesProcedureInflightor->cleanMessage($e->getMessage()), 500));
        }
        
	}
    
}