<?php

namespace Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Inflightor\Util\MessagesProcedureInflightor;
use Zeedhi\Framework\DTO\Response\Error;
use Doctrine\DBAL\Connection;

class UtilizacaoInsumosController extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = "";
    protected $dataSourceNameReceita = "utilrecflex";
	protected $dataSourceNameComposicao = "utilrecflexopcoes";
    
    const DELETE_CONFIG_COMP = "DELETE FROM UTILRECFLEXOPCOES WHERE CDFILIAL = :CDFILIAL AND CDRECEITA = :CDRECEITA AND NRSEQUTILRECFLEX = :NRSEQUTILRECFLEX";
    const VALIDA_VIGENCIA = "SELECT CDFILIAL, CDRECEITA, NRSEQUTILRECFLEX, DTINIVIGENCIA, DTFIMVIGENCIA, NRORG FROM UTILRECFLEX WHERE CDFILIAL = :CDFILIAL AND CDRECEITA = :CDRECEITA AND NRORG = :NRORG AND( (DTINIVIGENCIA BETWEEN :DTINIVIGENCIA AND :DTFIMVIGENCIA) OR(DTFIMVIGENCIA BETWEEN :DTINIVIGENCIA AND :DTFIMVIGENCIA) OR(:DTFIMVIGENCIA BETWEEN DTINIVIGENCIA AND DTFIMVIGENCIA) OR(:DTINIVIGENCIA BETWEEN DTINIVIGENCIA AND DTFIMVIGENCIA)) AND NRSEQUTILRECFLEX <> NVL(:NRSEQUTILRECFLEX, -1)";

    public function __construct($crudService, $parameterBag,  $environment, $dataSourceManager, $entityManager) {
        $this->crudService = $crudService;
        $this->parameterBag=$parameterBag;
        $this->environment=$environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager = $entityManager;
    }

    private function getComposicao($cdreceita, $nrvariacaorec){
    	$conditions = array(
            array(
                'columnName' => 'CDRECEITA',
                'operator' => '=',
                'value' => $cdreceita
            ),
            array(
                'columnName' => 'NRVARIACAOREC',
                'operator' => '=',
                'value' => $nrvariacaorec
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrgTrab()
            ),
            array(
                'columnName' => 'IDPREFERENCIA',
                'operator' => '=',
                'value' => 'S'
            )
        );
        return $this->crudService->getFilteredDataSet('coc01400_receitas_produtos', $conditions);
    }

    private function getVariacao($cdfilial, $cdreceita){
        $conditions = array(
            array(
                'columnName' => 'CDFILIAL',
                'operator' => '=',
                'value' => $cdfilial
            ),
            array(
                'columnName' => 'CDRECEITA',
                'operator' => '=',
                'value' => $cdreceita
            ),
            array(
                'columnName' => 'NRORG',
                'operator' => '=',
                'value' => $this->environment->getNrOrgTrab()
            )
        );
        return $this->crudService->getFilteredDataSet('filireceita', $conditions);
    }

    public function findReceitas(DTO\Request $request, DTO\Response $response){
        $this->dataSourceName = 'pcp00020_receitas_cadastradas';
        $this->parameterBagColumns = array("CDFILIAL", "DTINIVIGENCIA", "DTFIMVIGENCIA");
        parent::find($request, $response);
    }

    private function createDataComposicao($cdfilial, $cdreceita, $nrsequtilrecflex, $data){
        $dataSave = array();
        $nrorg = $this->environment->getNrOrgTrab();
        $cdoperador = $this->environment->getCdOperador();
        $date = $this->environment->getCurrentTime();
        foreach ($data->getRows() as $rowComposicao) {
            $row = array(
                "CDFILIAL" => $cdfilial,
                "CDRECEITA" => $cdreceita,
                "NRSEQUTILRECFLEX" => $nrsequtilrecflex,
                "NRSEQUTILRECFLEXOPCOES" => null,
                "CDPRODUTO" => $rowComposicao["ID"] == "P" ? $rowComposicao['CODIGO'] : null,
                "CDRECEITAOP" => $rowComposicao["ID"] == "S" ? $rowComposicao['CODIGO'] : null,
                "NRORG" => $nrorg,
                "DTINCLUSAO" => $date->format('d/m/Y h:m:s'),
                "CDOPERINCLUSAO" => $cdoperador,
                "DTULTATU" => $date->format('d/m/Y h:m:s'),
                "NRORGINCLUSAO" => $nrorg,
                "NRORGULTATU" => $nrorg,
                "CDOPERULTATU" => $cdoperador, 
                "IDATIVO" => 'S',
                "changed" => true,
                "__is_new" => true,
                "__new_code_length" => 10,
                "__new_code_pk" => "UTILRECFLEXOPCOES".$cdfilial.$cdreceita.$nrsequtilrecflex,
                "__table_pk" => "NRSEQUTILRECFLEXOPCOES",
                "__using_unit" => false
            ); 
            array_push($dataSave, $row);  
        }
        return $dataSave;
    }

	public function saveReceita(DTO\Request\Row $request, DTO\Response $response) {
		$connection = $this->entityManager->getConnection();
		try {
			$connection->beginTransaction();
            //transaction
                $row = $request->getRow();
                //Inclui a Receita com a nova vigência
                $this->_validaVigencia($connection, $row);
                $this->dataSourceName = $this->dataSourceNameReceita;
                $rows = $this->crudService->getRowsToSave($request, $this->dataSourceName);
                $this->crudService->save($this->dataSourceName, $rows);
                $callbackSequencial = $this->crudService->getColumnsToReturn();
                //Salva a configurãção de produção padrão da Receita
                //recebe a composição do cadastro de receita e utiliza como base inicial
                $variacao = $this->getVariacao($row['CDFILIAL'], $row['CDRECEITA']);
                if(!empty($variacao->getRows())){
                    $dataSetComposicao = $this->getComposicao($row['CDRECEITA'], $variacao->getRows()[0]['NRVARIACAOREC']);
        			$dataSave = $this->createDataComposicao($row['CDFILIAL'], $row['CDRECEITA'], $callbackSequencial[0]['NRSEQUTILRECFLEX'], $dataSetComposicao);
                    //Salva composição padrão de acordo com o Cadastro da Receita
                    $this->dataSourceName = $this->dataSourceNameComposicao;
                    $dataSetComp = new DataSet($this->dataSourceName, $dataSave);
                    $requestComp = new DTO\Request\DataSet($dataSetComp, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
                    parent::save($requestComp, $response);
                }
            //commit
			$connection->commit();
		} catch (\Exception $e) {
			$connection->rollBack();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

    public function saveConfigProducao(DTO\Request\DataSet $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        try {
            //transaction
            $connection->beginTransaction();
                $this->dataSourceName = $this->dataSourceNameComposicao;
                $rows = $request->getDataSet()->getRows();
                //Deleta as configuração de produção atual
                $this->_deleteConfigProducao($connection, $rows[0]['CDFILIAL'], $rows[0]['CDRECEITA'], $rows[0]['NRSEQUTILRECFLEX']);
                //Salva nova configuração de produção
                $this->_saveConfigProducao($request, $response);
            //commit
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            $response->setError(new Error("Error na alteração da Composição". $e->getMessage(), 500));
        }
    }

    public function deleteVigencia(DTO\Request\Row $request, DTO\Response $response){
        $connection = $this->entityManager->getConnection();
        try {
            //transaction
            $connection->beginTransaction();
                $this->dataSourceName = $this->dataSourceNameComposicao;
                $row = $request->getRow();
                //Deleta as configuração de produção atual
                $this->_deleteConfigProducao($connection, $row['CDFILIAL'], $row['CDRECEITA'], $row['NRSEQUTILRECFLEX']);
                $this->dataSourceName = $this->dataSourceNameReceita;
                parent::delete($request, $response);
            //commit
            $connection->commit();
        } catch (\Exception $e){
            $connection->rollBack();
            $response->setError(new Error("Error na alteração da Composição". $e->getMessage(), 500));
        }
    }

    private function _deleteConfigProducao($connection, $cdfilial, $cdreceita, $nrsequtilrecflex){
        $stmt = $connection->prepare(SELF::DELETE_CONFIG_COMP);
        $stmt->execute(array(
            "CDFILIAL" => $cdfilial,
            "CDRECEITA" => $cdreceita,
            "NRSEQUTILRECFLEX" => $nrsequtilrecflex
        ));
    }

    private function _saveConfigProducao($request, $response){
        $rows = $request->getDataSet()->getRows();
        $dataSave = $this->createDataComposicao($rows[0]['CDFILIAL'], $rows[0]['CDRECEITA'], $rows[0]['NRSEQUTILRECFLEX'], $request->getDataSet());
        //Salva composição padrão de acordo com o Cadastro da Receita
        $this->dataSourceName = $this->dataSourceNameComposicao;
        $dataSetComp = new DataSet($this->dataSourceName, $dataSave);
        $requestComp = new DTO\Request\DataSet($dataSetComp, $request->getMethod(), $request->getRoutePath(), $request->getUserId());
        parent::save($requestComp, $response);
    }

    private function _validaVigencia($connection, $row) {
        $cdfilial = $row['CDFILIAL'];
        $cdreceita = $row['CDRECEITA'];
        $nrorg = $row['NRORG'];
        $dtinivigencia = $row['DTINIVIGENCIA'];
        $dtfimvigencia = $row['DTFIMVIGENCIA'];
        $nrsequtilrecflex = array_key_exists('NRSEQUTILRECFLEX', $row) ? $row['NRSEQUTILRECFLEX'] : null;
        $stmt = $connection->prepare(SELF::VALIDA_VIGENCIA);
        $stmt->execute(
            array(
                "CDFILIAL" => $cdfilial,
                "CDRECEITA" => $cdreceita,
                "NRORG" => $nrorg,
                "DTINIVIGENCIA" => $dtinivigencia,
                "DTFIMVIGENCIA" => $dtfimvigencia,
                "NRSEQUTILRECFLEX" => $nrsequtilrecflex
            )
        );
        $result = $stmt->fetch(\PDO::FETCH_ASSOC);
        if($result)
            throw new \Exception("Período de vigência já cadastrado.", 1);
    }
}